"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compose = exports.constant = exports.enumerate = exports.nillable = exports.optional = exports.nullable = exports.anyOf = exports.oneOf = exports.map = exports.array = exports.ref = exports.object = exports.relativeJsonPointer = exports.jsonPointer = exports.uuid = exports.regex = exports.ipv6 = exports.ipv4 = exports.hostname = exports.email = exports.uriTemplate = exports.uriReference = exports.uri = exports.duration = exports.dateTime = exports.time = exports.date = exports.anonymousData = exports.any = exports.boolean = exports.number = exports.string = void 0;
const string = (options = {}) => ({
    type: 'string',
    ...options,
});
exports.string = string;
const number = (options = {}) => ({
    type: 'number',
    ...options,
});
exports.number = number;
const boolean = (options = {}) => ({
    type: 'boolean',
    ...options,
});
exports.boolean = boolean;
const any = (options = {}) => ({
    ...options,
});
exports.any = any;
const anonymousData = (options) => ({
    additionalProperties: { type: 'string' },
    ...options,
});
exports.anonymousData = anonymousData;
const stringFormat = (format) => (options = {}) => ({
    type: 'string',
    format,
    ...(options ?? {}),
});
exports.date = stringFormat('date');
exports.time = stringFormat('time');
exports.dateTime = stringFormat('date-time');
exports.duration = stringFormat('duration');
exports.uri = stringFormat('uri');
exports.uriReference = stringFormat('uri-reference');
exports.uriTemplate = stringFormat('uri-template');
exports.email = stringFormat('email');
exports.hostname = stringFormat('hostname');
exports.ipv4 = stringFormat('ipv4');
exports.ipv6 = stringFormat('ipv6');
exports.regex = stringFormat('regex');
exports.uuid = stringFormat('uuid');
exports.jsonPointer = stringFormat('json-pointer');
exports.relativeJsonPointer = stringFormat('relative-json-pointer');
const object = (properties, options = {}) => {
    const required = [];
    const schemaProperties = {};
    Object.entries(properties).forEach(([key, property]) => {
        if (property.kind === 'custom') {
            if (!property.optional) {
                required.push('key');
            }
            schemaProperties[key] = property.object;
        }
        else {
            required.push(key);
            schemaProperties[key] = property;
        }
    });
    return {
        type: 'object',
        properties: schemaProperties,
        required: required.length === 0 ? undefined : required,
        ...options,
    };
};
exports.object = object;
const ref = (refName) => ({
    $ref: `#/definitions/${refName}`,
});
exports.ref = ref;
const autoRef = (type) => (typeof type === 'string' ? (0, exports.ref)(type) : type);
const array = (itemType, options = {}) => ({
    type: 'array',
    items: autoRef(itemType),
    ...options,
});
exports.array = array;
const map = (itemType) => ({
    type: 'object',
    patternProperties: {
        '.*': autoRef(itemType),
    },
    additionalProperties: false,
});
exports.map = map;
const oneOf = (types) => ({
    oneOf: types.map(autoRef),
});
exports.oneOf = oneOf;
const anyOf = (types) => ({
    anyOf: types.map(autoRef),
});
exports.anyOf = anyOf;
const nullable = (type) => {
    const obj = autoRef(type);
    const types = ['string', 'number', 'boolean', 'integer'];
    if (typeof obj.type === 'string' && types.includes(obj.type)) {
        return {
            ...obj,
            type: [obj.type, 'null'],
        };
    }
    return (0, exports.anyOf)([obj, { type: 'null' }]);
};
exports.nullable = nullable;
const optional = (type) => ({
    kind: 'custom',
    object: autoRef(type),
    optional: true,
});
exports.optional = optional;
const nillable = (type) => (0, exports.optional)((0, exports.nullable)(type));
exports.nillable = nillable;
const enumerate = (values) => ({
    type: 'string',
    enum: values,
});
exports.enumerate = enumerate;
const constant = (value) => ({
    type: 'string',
    enum: [value],
});
exports.constant = constant;
const compose = (...sources) => {
    if (sources === undefined) {
        throw new Error(`Sources for 'compose' cannot be undefined`);
    }
    const properties = {};
    const requiredRecord = {};
    sources.forEach(source => {
        Object.assign(properties, source.properties);
        if (Array.isArray(source.required)) {
            source.required.forEach(key => {
                requiredRecord[key] = true;
            });
        }
    });
    const required = Object.keys(requiredRecord);
    return {
        type: 'object',
        properties,
        required: required.length === 0 ? undefined : required,
    };
};
exports.compose = compose;
