"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateAjvValidator = void 0;
const prettier_1 = require("prettier");
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const helpersTemplate = `
/* eslint-disable */
import type { ErrorObject } from 'ajv';
export interface Validator {
  (json: unknown): boolean;
  errors?: ErrorObject[] | null;
}
export function validateJson(json: any, validator: Validator, definitionName: string): any {
  const jsonObject = typeof json === 'string' ? JSON.parse(json) : json;
  if (validator(jsonObject)) {
    return jsonObject;
  }
  const jsonPreviewStr = (typeof json === 'string' ? json : JSON.stringify(jsonObject)).substring(0, 200);
  if (validator.errors) {
    throw Error(\`\${definitionName} \${errorsText(validator.errors)}. JSON: \${jsonPreviewStr}\`);
  }
  throw Error(\`\${definitionName} Unexpected data received. JSON: \${jsonPreviewStr}\`);
}
function errorsText(errors: ErrorObject[]): string {
  return errors.map(error => \`\${error.instancePath}: \${error.message}\`).join('\\n')
}
`;
function generateAjvValidator(prettierOptions, outDirs) {
    const helpers = (0, prettier_1.format)(helpersTemplate, prettierOptions);
    outDirs.forEach((outDir) => {
        (0, fs_1.mkdirSync)(outDir, { recursive: true });
        (0, fs_1.writeFileSync)(path_1.default.join(outDir, `validate.ts`), helpers);
    });
}
exports.generateAjvValidator = generateAjvValidator;
