"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateMetaFile = void 0;
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const prettier_1 = require("prettier");
const metaTemplate = (esm) => {
    const importExtension = esm ? '.js' : '';
    return `
  /* eslint-disable */
  import { $ModelImports } from './models${importExtension}';
  export const schemaDefinitions = {
    $Definitions
  }
  export interface SchemaInfo<T> {
    definitionName: string;
    schemaRef: string;
  }
  function info<T>(definitionName: string, schemaRef: string): SchemaInfo<T> {
    return { definitionName, schemaRef };
  }
  `;
};
function generateMetaFile(definitionNames, outDirs, prettierOptions, esm) {
    const metas = definitionNames
        .map(definitionName => `${definitionName}: info<${definitionName}>('${definitionName}', '#/definitions/${definitionName}'),`)
        .join('\n');
    const rawOutput = metaTemplate(esm)
        .replace(/\$Definitions/g, metas)
        .replace(/\$ModelImports/g, definitionNames.join(', '));
    const output = (0, prettier_1.format)(rawOutput, prettierOptions);
    outDirs.forEach(outDir => {
        (0, fs_1.mkdirSync)(outDir, { recursive: true });
        (0, fs_1.writeFileSync)(path_1.default.join(outDir, `meta.ts`), output);
    });
}
exports.generateMetaFile = generateMetaFile;
