"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pathsToControllersRoutes = void 0;
function convertSchema(route) {
    const schema = {};
    if (route.requestBody && 'content' in route.requestBody) {
        schema.body = route.requestBody.content['application/json'].schema;
    }
    if (route.parameters) {
        const params = route.parameters;
        console.log('params', params);
    }
    return schema;
}
function convertRouteInfo(groupPath, route) {
    const results = [];
    ['get', 'put', 'post', 'patch', 'delete'].forEach(m => {
        const method = m.toUpperCase();
        if (m in route) {
            results.push({
                method,
                schema: convertSchema(route[m]),
                summary: route[m]?.summary,
                url: groupPath,
            });
        }
    });
    if (!results.length)
        throw new Error(`No methods found in route${JSON.stringify(route)}`);
    return results;
}
function extractGroupFromRoutePath(path) {
    if (!path)
        return null;
    const pathWithoutFirstSlash = path[0] === '/' ? path.slice(1) : path;
    const firstIndexOfSlash = pathWithoutFirstSlash.indexOf('/');
    const group = pathWithoutFirstSlash.slice(0, firstIndexOfSlash);
    const groupPath = pathWithoutFirstSlash.slice(firstIndexOfSlash);
    return { group, groupPath };
}
async function pathsToControllersRoutes(schema) {
    const routeGroups = new Map();
    const pathNames = Object.keys(schema.paths);
    if (pathNames.length === 0)
        throw new Error('No paths found.');
    pathNames.forEach(pathName => {
        const group = extractGroupFromRoutePath(pathName);
        if (!group)
            throw new Error(`Cant extract group from route path ${pathName}`);
        routeGroups.set(group.group, convertRouteInfo(group.groupPath, schema.paths[pathName]));
    });
    return routeGroups;
}
exports.pathsToControllersRoutes = pathsToControllersRoutes;
