#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const yargs_1 = __importDefault(require("yargs"));
const new_generators_1 = require("./new-generators");
const parse_controller_schema_1 = require("./parse-controller-schema");
const parse_yaml_schema_1 = require("./parse-yaml-schema");
const utils_1 = require("./utils");
// TODO: clone https://github.com/Q42/openapi-typescript-validator
const options = (0, yargs_1.default)(process.argv.slice(2))
    .options({
    root: { type: 'string', demandOption: true },
    out: { type: 'string', demandOption: true },
    prettierrc: { type: 'string', demandOption: true },
})
    .parseSync();
if (path_1.default.extname(options.root) !== '.yaml' || !fs_1.default.lstatSync(options.root).isFile()) {
    throw new Error(`Must provide absolute path to index.yaml file, given: ${options.root}`);
}
if (!fs_1.default.lstatSync(options.prettierrc).isFile()) {
    throw new Error(`Must provide a valid path to .prettierrc file, given: ${options.prettierrc}`);
}
async function main() {
    const pathToIndex = (0, utils_1.resolvePath)(options.root);
    const pathToPrettierrc = (0, utils_1.resolvePath)(options.prettierrc);
    const schema = await (0, parse_yaml_schema_1.parseSchema)(pathToIndex);
    const controllerSchema = await (0, parse_controller_schema_1.pathsToControllersRoutes)(schema);
    // eslint-disable-next-line import/no-dynamic-require, global-require
    const prettierConfig = require(pathToPrettierrc);
    await (0, new_generators_1.generate)({
        outDirectory: (0, utils_1.resolvePath)(options.out),
        schemeInterfaces: {},
        prettierOptions: {
            parser: 'typescript',
            ...prettierConfig,
            plugins: [],
        },
    }, controllerSchema);
}
main().catch(console.error);
// TODO:
// 1) async resolve refs
// 2) generate:
// controllers:
/**
 * paths:
  /common/files/download-protected:
    $ref: './common/paths.yaml#/FilesDownloadProtected'

  /auth/login:
    $ref: './auth/paths.yaml#/Login'
  /auth/logout:
    $ref: './auth/paths.yaml#/Logout'
  /auth/refresh:
    $ref: './auth/paths.yaml#/Refresh'
  /auth/current-user:
    $ref: './auth/paths.yaml#/CurrentUser'
 */
/**
 * results to:
 * export class LoginRequest {
 *   static parser = (new Ajv({
 *      coerceTypes: true,
 *   })).compile({
 *      properties: []
 *          { name: 'login', type: 'string' },
 *          { name: 'password', type: 'string' },
 *      },
 *      required: ['username', 'password']
 *   });
 *
 *   data: ReturnType<typeof ajv.parse>;
 *
 *   constructor(from: object) {
 *     this.data = LoginRequest.parser(from);
 *   }
 * }
 *
 * export class AuthController {
 *    \/**
 *      Вход в систему по логину/паролю
 *    *\/
 *    async login(body: ) {
 *    }
 *    async logout() {
 *    }
 *    async refresh() {
 *    }
 *    async currentUser() {
 *    }
 * }
 */
// routes
// generate({
//     schemaFile: options.root,
//     schemaType: 'yaml',
//     directory: options.out,
//     // standalone: {
//     //     validatorOutput: 'commonjs',
//     //     mergeDecoders: true,
//     // }
// }).then(() => {
//     console.log('✅ Generated files to', options.out);
// });
