"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateInterface = exports.getRouteInterfaceNames = void 0;
const json_schema_to_typescript_1 = require("json-schema-to-typescript");
const utils_1 = require("../utils");
const getRouteInterfaceNames = (prefix, route) => {
    const name = (0, utils_1.extractNameFromPath)(prefix + route.operationId);
    return {
        body: `${name}Body`,
        params: `${name}Params`,
        query: `${name}Query`,
        response: `${name}Response`,
    };
};
exports.getRouteInterfaceNames = getRouteInterfaceNames;
async function generateInterface(options, schema) {
    const interfaces = new Map();
    const groupNames = [...schema.keys()];
    await Promise.all(groupNames.map(async (groupName) => {
        const groupRoutes = schema.get(groupName);
        interfaces.set(groupName, []);
        await Promise.all(groupRoutes.map(async (route) => {
            const prefix = options.schemeInterfaces.groupPrefix ? route.group : '';
            const names = (0, exports.getRouteInterfaceNames)(prefix, route);
            const routeTypes = {
                names,
            };
            if (route.schema.body) {
                const ts = await (0, json_schema_to_typescript_1.compile)(route.schema.body, names.body, {
                    bannerComment: '',
                    additionalProperties: false,
                });
                routeTypes.body = ts;
            }
            if (Object.keys(route.schema.params).length > 0) {
                const ts = await (0, json_schema_to_typescript_1.compile)(route.schema.params, names.params, {
                    bannerComment: '',
                    additionalProperties: false,
                });
                routeTypes.params = ts;
            }
            if (Object.keys(route.schema.querystring).length > 0) {
                const ts = await (0, json_schema_to_typescript_1.compile)(route.schema.querystring, names.query, {
                    bannerComment: '',
                    additionalProperties: false,
                });
                routeTypes.query = ts;
            }
            if (route.schema.response) {
                const responseSchema = {
                    oneOf: Object.values(route.schema.response)
                        .map((e) => {
                        if (e?.content?.['application/json'])
                            return e?.content?.['application/json'].schema;
                        if (e?.content?.['application/octet-stream'])
                            return e?.content?.['application/octet-stream']?.schema;
                        return undefined;
                    })
                        .filter(Boolean),
                };
                const ts = await (0, json_schema_to_typescript_1.compile)(responseSchema, names.response, {
                    bannerComment: '',
                    additionalProperties: false,
                });
                routeTypes.response = ts;
            }
            interfaces.get(groupName).push(routeTypes);
        }));
    }));
    return interfaces;
}
exports.generateInterface = generateInterface;
