"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateRoutes = void 0;
const imports_1 = require("./imports");
const schemas_1 = require("./schemas");
async function generateRoutes(options, schema) {
    const groupCodes = new Map();
    const groupNames = [...schema.keys()];
    await Promise.all(groupNames.map(async (groupName) => {
        const groupRoutes = schema.get(groupName);
        let inClassBody = '';
        const imports = new imports_1.ImportsGenerator();
        groupRoutes.map(async (route) => {
            const prefix = options.schemeInterfaces.groupPrefix ? route.group : '';
            const schemaObjectName = (0, schemas_1.getRouteSchemaObjectName)(prefix, route);
            const schemaTypeName = (0, schemas_1.getRouteSchemaTypeName)(prefix, route);
            imports.addImport('fastify', 'FastifyRequest');
            imports.addImport('fastify', 'FastifyReply');
            imports.addImport('fastify-decorators', route.method.toUpperCase());
            imports.addImport('./schemas', schemaObjectName);
            imports.addImport('./schemas', schemaTypeName);
            inClassBody += `
                    @${route.method.toUpperCase()}({
                        url: '${route.url}',
                        options: {
                            schema: ${schemaObjectName},
                        }
                    })
                    async ${route.operationId}Handler(req: FastifyRequest<${schemaTypeName}>, res: FastifyReply) {
                        // TODO: user code goes here
                        throw new Error('Not implemented');
                    }
                `;
        });
        groupCodes.set(groupName, {
            imports,
            inClassBody: inClassBody,
        });
    }));
    return groupCodes;
}
exports.generateRoutes = generateRoutes;
