"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSchemas = exports.getRouteSchemaObjectName = exports.getRouteSchemaTypeName = void 0;
const utils_1 = require("../utils");
const imports_1 = require("./imports");
const interfaces_1 = require("./interfaces");
const getRouteSchemaTypeName = (prefix, route) => `${(0, utils_1.ucFirst)(prefix) + (0, utils_1.ucFirst)(route.operationId)}SchemaType`;
exports.getRouteSchemaTypeName = getRouteSchemaTypeName;
const getRouteSchemaObjectName = (prefix, route) => `${prefix}${(0, utils_1.ucFirst)(route.operationId)}Schema`;
exports.getRouteSchemaObjectName = getRouteSchemaObjectName;
async function generateSchemas(options, schema) {
    const groupSchemas = new Map();
    const groupNames = [...schema.keys()];
    await Promise.all(groupNames.map(async (groupName) => {
        const groupRoutes = schema.get(groupName);
        groupSchemas.set(groupName, {
            imports: '',
            routes: [],
        });
        const imports = new imports_1.ImportsGenerator();
        await Promise.all(groupRoutes.map(async (route) => {
            const prefix = options.schemeInterfaces.groupPrefix ? route.group : '';
            const names = (0, interfaces_1.getRouteInterfaceNames)(prefix, route);
            const bodyInterface = route.schema.body ? names.body : undefined;
            const paramsInterface = Object.keys(route.schema.params).length > 0 ? names.params : undefined;
            const queryInterface = Object.keys(route.schema.querystring).length > 0 ? names.query : undefined;
            const responseInterface = route.schema.response ? names.response : undefined;
            const schemaTypeName = (0, exports.getRouteSchemaTypeName)(prefix, route);
            const schemaObjectName = (0, exports.getRouteSchemaObjectName)(prefix, route);
            const schemaObjectContent = JSON.stringify({ ...route.schema, response: undefined });
            const schemaTypeContent = `{
                        ${bodyInterface ? `Body: ${bodyInterface};` : ''}
                        ${paramsInterface ? `Params: ${paramsInterface};` : ''}
                        ${queryInterface ? `Querystring: ${queryInterface};` : ''}
                        ${responseInterface ? `Reply: ${responseInterface};` : ''}
                    }`;
            [bodyInterface, paramsInterface, queryInterface, responseInterface]
                .filter(Boolean)
                .forEach(interfaceName => imports.addImport('./types', interfaceName));
            groupSchemas.get(groupName).routes.push({
                schemaObjectContent,
                schemaObjectName,
                schemaTypeName,
                schemaTypeContent,
            });
        }));
        groupSchemas.get(groupName).imports = imports.generate();
    }));
    return groupSchemas;
}
exports.generateSchemas = generateSchemas;
