"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.pathsToControllersRoutes = void 0;
const openapi_schema_to_json_schema_1 = __importDefault(require("@openapi-contrib/openapi-schema-to-json-schema"));
function convertSchema(route) {
    const schema = {
        querystring: {
            type: 'object',
        },
        params: {
            type: 'object',
        },
    };
    if (route.requestBody && 'content' in route.requestBody) {
        const jsonType = 'application/json';
        const fileType = 'multipart/form-data';
        if (jsonType in route.requestBody.content) {
            schema.body = route.requestBody.content[jsonType].schema;
        }
        else if (fileType in route.requestBody.content) {
            // TODO: file upload validation support
        }
    }
    if (route.parameters) {
        const params = route.parameters;
        const mapping = {
            query: 'querystring',
            path: 'params',
        };
        params.forEach(param => {
            if (!(param.in in mapping))
                throw new Error(`Unknown \`in\` ${param.in} of parameter ${JSON.stringify(param)} of route ${JSON.stringify(route)}`);
            const schemaField = schema[mapping[param.in]];
            if (!schemaField.properties)
                schemaField.properties = {};
            schemaField.properties[param.name] = (0, openapi_schema_to_json_schema_1.default)(param.schema);
        });
    }
    if (route.responses) {
        const responseCodes = Object.keys(route.responses);
        if (responseCodes.length) {
            schema.response = route.responses;
        }
    }
    return schema;
}
function convertRouteInfo(group, groupPath, route) {
    const results = [];
    ['get', 'put', 'post', 'patch', 'delete'].forEach(m => {
        const method = m.toUpperCase();
        if (m in route) {
            results.push({
                method,
                schema: convertSchema(route[m]),
                summary: route[m]?.summary,
                url: groupPath,
                group,
            });
        }
    });
    if (!results.length)
        throw new Error(`No methods found in route${JSON.stringify(route)}`);
    return results;
}
function extractGroupFromRoutePath(path) {
    if (!path)
        return null;
    const pathWithoutFirstSlash = path[0] === '/' ? path.slice(1) : path;
    const firstIndexOfSlash = pathWithoutFirstSlash.indexOf('/');
    const group = pathWithoutFirstSlash.slice(0, firstIndexOfSlash);
    const groupPath = pathWithoutFirstSlash.slice(firstIndexOfSlash);
    return { group, groupPath };
}
async function pathsToControllersRoutes(schema) {
    const routeGroups = new Map();
    const pathNames = Object.keys(schema.paths);
    if (pathNames.length === 0)
        throw new Error('No paths found.');
    await Promise.all(pathNames.map(async (pathName) => {
        const group = extractGroupFromRoutePath(pathName);
        if (!group)
            throw new Error(`Cant extract group from route path ${pathName}`);
        if (!routeGroups.has(group.group)) {
            routeGroups.set(group.group, []);
        }
        const groupRoutes = convertRouteInfo(group.group, group.groupPath, schema.paths[pathName]);
        routeGroups.get(group.group).push(...groupRoutes);
    }));
    return routeGroups;
}
exports.pathsToControllersRoutes = pathsToControllersRoutes;
