"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fileExists = exports.extractNameFromPath = exports.ucFirst = exports.resolvePath = exports.isHTTPMethod = void 0;
const promises_1 = require("fs/promises");
const path_1 = require("path");
function isHTTPMethod(str) {
    return ['POST', 'GET', 'PATCH', 'PUT', 'DELETE'].includes(str);
}
exports.isHTTPMethod = isHTTPMethod;
const resolvePath = (absoluteOrRelativePath, relativeTo = process.cwd()) => {
    if ((0, path_1.isAbsolute)(absoluteOrRelativePath))
        return absoluteOrRelativePath;
    return (0, path_1.resolve)(relativeTo, absoluteOrRelativePath).replace(/\\/g, '/');
};
exports.resolvePath = resolvePath;
const ucFirst = (str) => str.charAt(0).toUpperCase() + str.slice(1);
exports.ucFirst = ucFirst;
function extractNameFromPath(path) {
    const pathWithoutFirstSlash = path[0] === '/' ? path.slice(1) : path;
    const trimmedPath = pathWithoutFirstSlash[pathWithoutFirstSlash.length - 1] === '/'
        ? pathWithoutFirstSlash.slice(0, pathWithoutFirstSlash.length - 1)
        : pathWithoutFirstSlash;
    const pathConverted = trimmedPath.replace(/[{}()]/g, '').replace(/-/g, '/');
    return pathConverted.split('/').map(exports.ucFirst).join('');
}
exports.extractNameFromPath = extractNameFromPath;
async function fileExists(path) {
    try {
        await (0, promises_1.access)(path);
        return true;
    }
    catch {
        return false;
    }
}
exports.fileExists = fileExists;
